CREATE TABLE T_Common (
  C_ID INT NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_Timezone SMALLINT NOT NULL,
  C_FileFlag NVARCHAR(64) NOT NULL,
  C_MajorVer INT NOT NULL,
  C_MinorVer INT NOT NULL,
  C_RevisionVer INT NOT NULL,
  C_CreateOK BOOLEAN NOT NULL,
  CONSTRAINT PK_Common PRIMARY KEY (C_ID));


CREATE TABLE T_User (
  C_ID INT NOT NULL, 
  C_Name NVARCHAR(64) NOT NULL,
  C_Password NVARCHAR(256),
  C_FullName NVARCHAR(256), 
  C_Email NVARCHAR(256),
  C_Department NVARCHAR(256),
  C_JobTitle NVARCHAR(256),
  C_Active BOOLEAN NOT NULL,
  C_Super BOOLEAN NOT NULL,
  C_ReadOnly BOOLEAN NOT NULL,  
  C_CanEditFolder BOOLEAN NOT NULL,
  C_CanEditLabelType BOOLEAN NOT NULL,
  C_Rights INT NOT NULL,
  C_Level INT NOT NULL,
  C_Setting NTEXT,
  CONSTRAINT PK_User PRIMARY KEY (C_ID),
  CONSTRAINT I_User_Name UNIQUE (C_Name));


CREATE TABLE T_GlobalFileOptions (
  C_ID INT NOT NULL, 
  C_GeneralFileOptions NTEXT,
  C_OtherOptions BLOB,
  C_ModuleOptions BLOB,
  C_Categories NTEXT,
  C_CustomFields NTEXT,
  CONSTRAINT PK_GlobalFileOptions PRIMARY KEY (C_ID));


CREATE TABLE T_SyncInfo (
  C_ID INT NOT NULL,
  C_From GUID NOT NULL,
  C_To GUID NOT NULL,
  C_LastSyncTime DATETIME NOT NULL,
  C_SyncVersion INT NOT NULL,
  C_SyncInfo BLOB, 
  CONSTRAINT PK_SyncInfo PRIMARY KEY (C_ID));


CREATE TABLE T_DeleteLog (
  C_InfoTypeID INT NOT NULL,
  C_InfoGUID GUID NOT NULL,
  C_DeletedTime BIGINT NOT NULL);


CREATE INDEX I_DeleteLog_ITID_DT ON T_DeleteLog (C_InfoTypeID, C_DeletedTime);


CREATE TABLE T_FileOptions (
  C_ID INT NOT NULL, 
  C_UserID INT NOT NULL, 
  C_GeneralFileOptions NTEXT, 
  C_CalendarTaskPaneGVSetting NTEXT, 
  C_GlobalSearchResultGVSetting NTEXT, 
  CONSTRAINT PK_FileOptions PRIMARY KEY (C_ID),
  CONSTRAINT FK_FileOptions_UserID FOREIGN KEY (C_UserID) REFERENCES T_User ON DELETE CASCADE);


CREATE TABLE T_InfoType (
  C_ID INT NOT NULL, 
  C_InfoTypeGUID GUID NOT NULL,
  CONSTRAINT PK_InfoType PRIMARY KEY (C_ID),
  CONSTRAINT I_InfoType_InfoTypeGUID UNIQUE (C_InfoTypeGUID));


CREATE TABLE T_Calendar (
  C_ID INT NOT NULL, 
  C_CalendarGUID GUID NOT NULL, 
  C_IsGregorianCalendar BOOLEAN NOT NULL,
  CONSTRAINT PK_Calendar PRIMARY KEY (C_ID),
  CONSTRAINT I_Calendar_CalendarGUID UNIQUE (C_CalendarGUID));


CREATE TABLE T_Folder (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_ParentID INT,
  C_InfoTypeID INT NOT NULL,
  C_Order INT NOT NULL,
  C_Name NVARCHAR(256), 
  C_IsDefault BOOLEAN NOT NULL,
  C_Visible BOOLEAN DEFAULT 1 NOT NULL,
  CONSTRAINT PK_Folder PRIMARY KEY (C_ID),
  CONSTRAINT FK_Folder_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Folder_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Folder_InfoTypeID FOREIGN KEY (C_InfoTypeID) REFERENCES T_InfoType,
  CONSTRAINT FK_Folder_ParentID FOREIGN KEY (C_ParentID) REFERENCES T_Folder ON DELETE SET NULL);


CREATE TABLE T_ViewSetting (
  C_ID INT NOT NULL,
  C_UserID INT NOT NULL, 
  C_InfoTypeID INT NOT NULL,
  C_FolderID INT,
  C_ViewSettingType SMALLINT NOT NULL, 
  C_GeneralViewSetting NTEXT, 
  C_GridViewSetting NTEXT,
  CONSTRAINT PK_ViewSetting PRIMARY KEY (C_ID),
  CONSTRAINT FK_ViewSetting_UserID FOREIGN KEY (C_UserID) REFERENCES T_User ON DELETE CASCADE,
  CONSTRAINT FK_ViewSetting_InfoTypeID FOREIGN KEY (C_InfoTypeID) REFERENCES T_InfoType,
  CONSTRAINT FK_ViewSetting_FolderID FOREIGN KEY (C_FolderID) REFERENCES T_Folder ON DELETE CASCADE);


CREATE TABLE T_InfoFolder (
  C_InfoTypeID INT NOT NULL,
  C_InfoID INT NOT NULL,
  C_FolderID INT NOT NULL,
  CONSTRAINT PK_InfoFolder PRIMARY KEY (C_InfoTypeID, C_InfoID, C_FolderID),
  CONSTRAINT FK_InfoFolder_InfoTypeID FOREIGN KEY (C_InfoTypeID) REFERENCES T_InfoType,
  CONSTRAINT FK_InfoFolder_FolderID FOREIGN KEY (C_FolderID) REFERENCES T_Folder ON DELETE CASCADE);


CREATE TABLE T_Attachment (
  C_ID INT NOT NULL,
  C_InfoTypeID INT NOT NULL,
  C_InfoID INT NOT NULL,
  C_URL NVARCHAR(256),
  C_Data BLOB,
  CONSTRAINT PK_Attachment PRIMARY KEY (C_ID),
  CONSTRAINT FK_Attachment_InfoTypeID FOREIGN KEY (C_InfoTypeID) REFERENCES T_InfoType);


CREATE TABLE T_Sound (
  C_ID INT NOT NULL,
  C_FileName NVARCHAR(256),
  C_URL NVARCHAR(256),
  C_Data BLOB,
  CONSTRAINT PK_Sound PRIMARY KEY (C_ID));


CREATE TABLE T_Contact (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_PhotoFlag BOOLEAN NOT NULL,
  C_Priority SMALLINT NOT NULL,
  C_Salutation NVARCHAR(256), 
  C_FirstName NVARCHAR(256), 
  C_MiddleName NVARCHAR(256), 
  C_LastName NVARCHAR(256), 
  C_FullName NVARCHAR(256), 
  C_FullNameOrder SMALLINT NOT NULL, 
  C_FullNameAbbr NVARCHAR(256), 
  C_Title NVARCHAR(256), 
  C_TitleOrder SMALLINT NOT NULL, 
  C_TitleAbbr NVARCHAR(256), 
  C_Nickname NVARCHAR(256), 
  C_Gender SMALLINT NOT NULL, 
  C_Birthday DATETIME, 
  C_BirthdayCalendarID INT NOT NULL,
  C_RemindBirthday BOOLEAN NOT NULL, 
  C_Anniversary DATETIME, 
  C_AnniversaryCalendarID INT NOT NULL,
  C_RemindAnniversary BOOLEAN NOT NULL, 
  C_Spouse NVARCHAR(256), 
  C_Family NVARCHAR(256), 
  C_Hobby NVARCHAR(256), 
  C_Specialty NVARCHAR(256), 
  C_Strength NVARCHAR(256), 
  C_Personality NVARCHAR(256), 
  C_MailingAddressType SMALLINT NOT NULL, 
  C_HomeStreet NVARCHAR(256), 
  C_HomeCity NVARCHAR(256), 
  C_HomeState NVARCHAR(256), 
  C_HomeZipcode NVARCHAR(256), 
  C_HomeCountry NVARCHAR(256), 
  C_HomePhone NVARCHAR(64), 
  C_HomePhone2 NVARCHAR(64), 
  C_HomePhone3 NVARCHAR(64), 
  C_HomePhone4 NVARCHAR(64), 
  C_HomeFax NVARCHAR(64), 
  C_PersonalWebPage NVARCHAR(256), 
  C_Company NVARCHAR(256), 
  C_JobTitle NVARCHAR(256), 
  C_Department NVARCHAR(256), 
  C_Office NVARCHAR(256), 
  C_Supervisor NVARCHAR(256), 
  C_Assistant NVARCHAR(256), 
  C_BusinessStreet NVARCHAR(256), 
  C_BusinessCity NVARCHAR(256), 
  C_BusinessState NVARCHAR(256), 
  C_BusinessZipcode NVARCHAR(256), 
  C_BusinessCountry NVARCHAR(256), 
  C_BusinessPhone NVARCHAR(64), 
  C_BusinessPhone2 NVARCHAR(64), 
  C_BusinessPhone3 NVARCHAR(64), 
  C_BusinessPhone4 NVARCHAR(64), 
  C_BusinessFax NVARCHAR(64), 
  C_BusinessWebPage NVARCHAR(256), 
  C_Email NVARCHAR(256), 
  C_Email2 NVARCHAR(256), 
  C_Email3 NVARCHAR(256), 
  C_Email4 NVARCHAR(256), 
  C_Mobile NVARCHAR(64), 
  C_Mobile2 NVARCHAR(64), 
  C_Mobile3 NVARCHAR(64), 
  C_Mobile4 NVARCHAR(64), 
  C_IM NVARCHAR(256), 
  C_IM2 NVARCHAR(256), 
  C_IM3 NVARCHAR(256), 
  C_IM4 NVARCHAR(256), 
  C_Custom1 NVARCHAR(256),
  C_Custom2 NVARCHAR(256),
  C_Custom3 NVARCHAR(256),
  C_Custom4 NVARCHAR(256),
  C_Custom5 NVARCHAR(256),
  C_Custom6 NVARCHAR(256),
  C_Custom7 NVARCHAR(256), 
  C_Custom8 NVARCHAR(256), 
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_Photos NTEXT,
  C_CommentPlainText NTEXT,
  C_Comment BLOB,
  C_MobileComment NTEXT,
  CONSTRAINT PK_Contact PRIMARY KEY (C_ID),
  CONSTRAINT FK_Contact_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Contact_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Contact_BirthdayCalendarID FOREIGN KEY (C_BirthdayCalendarID) REFERENCES T_Calendar,
  CONSTRAINT FK_Contact_ACID FOREIGN KEY (C_AnniversaryCalendarID) REFERENCES T_Calendar);


CREATE TABLE T_ContactPhoto (
  C_ID INT NOT NULL,
  C_ContactID INT NOT NULL,
  C_URL NVARCHAR(256),
  C_Data BLOB,
  CONSTRAINT PK_ContactPhoto PRIMARY KEY (C_ID));


CREATE TABLE T_InfoContact (
  C_InfoTypeID INT NOT NULL,
  C_InfoID INT NOT NULL,
  C_ContactID INT NOT NULL,
  CONSTRAINT PK_InfoContact PRIMARY KEY (C_InfoTypeID, C_InfoID, C_ContactID),
  CONSTRAINT FK_InfoContact_InfoTypeID FOREIGN KEY (C_InfoTypeID) REFERENCES T_InfoType,
  CONSTRAINT FK_InfoContact_ContactID FOREIGN KEY (C_ContactID) REFERENCES T_Contact ON DELETE CASCADE);


CREATE TABLE T_EventLabel (
  C_ID INT NOT NULL, 
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_Order INT NOT NULL,
  C_Name NVARCHAR(256), 
  C_LabelType INT NOT NULL,
  C_Color INT NOT NULL,
  CONSTRAINT PK_EventLabel PRIMARY KEY (C_ID));


CREATE TABLE T_Event (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_ShowOnCalendar BOOLEAN DEFAULT 1 NOT NULL, 
  C_HasSpecialItem BOOLEAN NOT NULL, 
  C_DeletedEventItems NTEXT,
  C_LabelID INT DEFAULT 1 NOT NULL,
  C_Subject NVARCHAR(256), 
  C_Location NVARCHAR(256), 
  C_StartTime DATETIME NOT NULL, 
  C_EndTime DATETIME NOT NULL, 
  C_AllDayFlag BOOLEAN NOT NULL, 
  C_Priority SMALLINT NOT NULL, 
  C_RemindType SMALLINT NOT NULL, 
  C_RemindBeforeMinutes INT, 
  C_RemindSpecialTime DATETIME, 
  C_RemindSound NVARCHAR(256),
  C_RemindActions NTEXT,
  C_RecurrencePattern SMALLINT NOT NULL, 
  C_RecurrenceFrequency INT, 
  C_RecurrenceSubPattern SMALLINT, 
  C_RecurrenceFestivalIdentifier INT, 
  C_CalendarID INT NOT NULL,
  C_Workdays SMALLINT NOT NULL, 
  C_FirstDayOfWeek SMALLINT NOT NULL, 
  C_RecurrenceRangeType SMALLINT, 
  C_RecurrenceStartDate DATETIME, 
  C_RecurrenceEndDate DATETIME, 
  C_RecurrenceTimes INT, 
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_CommentPlainText NTEXT,
  C_Comment BLOB,
  C_MobileComment NTEXT,
  CONSTRAINT PK_Event PRIMARY KEY (C_ID),
  CONSTRAINT FK_Event_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Event_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Event_LabelID FOREIGN KEY (C_LabelID) REFERENCES T_EventLabel ON DELETE SET DEFAULT,
  CONSTRAINT FK_Event_CalendarID FOREIGN KEY (C_CalendarID) REFERENCES T_Calendar);


CREATE TABLE T_ContactEvent (
  C_ContactID INT NOT NULL,
  C_EventID INT NOT NULL,
  C_AnniversaryType SMALLINT NOT NULL, 
  CONSTRAINT PK_ContactEvent PRIMARY KEY (C_ContactID, C_EventID, C_AnniversaryType),
  CONSTRAINT FK_ContactEvent_ContactID FOREIGN KEY (C_ContactID) REFERENCES T_Contact ON DELETE CASCADE,
  CONSTRAINT FK_ContactEvent_EventID FOREIGN KEY (C_EventID) REFERENCES T_Event ON DELETE CASCADE);


CREATE TABLE T_EventItem (
  C_ID INT NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_EventID INT NOT NULL,
  C_ScheduledTime DATETIME NOT NULL, 
  C_LabelID INT DEFAULT 1 NOT NULL,
  C_Subject NVARCHAR(256), 
  C_Location NVARCHAR(256), 
  C_StartTime DATETIME NOT NULL, 
  C_EndTime DATETIME NOT NULL, 
  C_AllDayFlag BOOLEAN NOT NULL, 
  C_Priority SMALLINT NOT NULL, 
  C_RemindType SMALLINT NOT NULL, 
  C_RemindBeforeMinutes INT, 
  C_RemindSpecialTime DATETIME, 
  C_RemindSound NVARCHAR(256),
  C_RemindActions NTEXT,
  C_CommentPlainText NTEXT,
  C_Comment BLOB, 
  C_MobileComment NTEXT,
  C_UseOwnLabel BOOLEAN NOT NULL, 
  C_UseOwnSubject BOOLEAN NOT NULL, 
  C_UseOwnLocation BOOLEAN NOT NULL, 
  C_UseOwnTimeSetting BOOLEAN NOT NULL, 
  C_UseOwnPriority BOOLEAN NOT NULL, 
  C_UseOwnRemindSetting BOOLEAN NOT NULL, 
  C_UseOwnRemindSound BOOLEAN NOT NULL, 
  C_UseOwnRemindActions BOOLEAN NOT NULL, 
  C_UseOwnComment BOOLEAN NOT NULL,
  C_UseOwnMobileComment BOOLEAN NOT NULL,
  CONSTRAINT PK_EventItem PRIMARY KEY (C_ID),
  CONSTRAINT FK_EventItem_EventID FOREIGN KEY (C_EventID) REFERENCES T_Event ON DELETE CASCADE,
  CONSTRAINT FK_EventItem_LabelID FOREIGN KEY (C_LabelID) REFERENCES T_EventLabel ON DELETE SET DEFAULT);


CREATE TABLE T_TaskLabel (
  C_ID INT NOT NULL, 
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_Order INT NOT NULL, 
  C_Name NVARCHAR(256), 
  C_LabelType INT NOT NULL,
  C_Color INT NOT NULL,
  CONSTRAINT PK_TaskLabel PRIMARY KEY (C_ID));


CREATE TABLE T_Task (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_AssignedTo INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_Order DOUBLE NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_ShowOnCalendar BOOLEAN DEFAULT 1 NOT NULL, 
  C_ParentID INT,
  C_ScheduledTime DATETIME, 
  C_RegularTaskID INT,
  C_LabelID INT DEFAULT 1 NOT NULL,
  C_Subject NVARCHAR(256), 
  C_Location NVARCHAR(256), 
  C_StartTime DATETIME, 
  C_EndTime DATETIME, 
  C_EndTimeIsNull BOOLEAN NOT NULL, 
  C_CompletionTime DATETIME, 
  C_AllDayFlag BOOLEAN NOT NULL, 
  C_Priority SMALLINT NOT NULL, 
  C_State SMALLINT NOT NULL, 
  C_CompletedFlag BOOLEAN NOT NULL, 
  C_CompletionPercent SMALLINT NOT NULL, 
  C_RemindType SMALLINT NOT NULL, 
  C_RemindBeforeMinutes INT, 
  C_RemindSpecialTime DATETIME, 
  C_RemindSound NVARCHAR(256),
  C_RemindActions NTEXT,
  C_FirstDueStartTime DATETIME, 
  C_FirstDueEndTime DATETIME, 
  C_RecurrencePattern SMALLINT NOT NULL, 
  C_RecurrenceFrequency INT, 
  C_RecurrenceSubPattern SMALLINT, 
  C_RecurrenceOnCompletionTime BOOLEAN NOT NULL, 
  C_RecurrenceFestivalIdentifier INT, 
  C_RecurrenceRemindType SMALLINT, 
  C_CalendarID INT NOT NULL,
  C_Workdays SMALLINT NOT NULL, 
  C_FirstDayOfWeek SMALLINT NOT NULL, 
  C_RecurrenceRangeType SMALLINT, 
  C_RecurrenceStartDate DATETIME, 
  C_RecurrenceEndDate DATETIME, 
  C_RecurrenceTimes INT, 
  C_RecurrenceLeftTimes INT, 
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_CommentPlainText NTEXT,
  C_Comment BLOB,
  C_MobileComment NTEXT,
  CONSTRAINT PK_Task PRIMARY KEY (C_ID),
  CONSTRAINT FK_Task_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Task_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Task_LabelID FOREIGN KEY (C_LabelID) REFERENCES T_TaskLabel ON DELETE SET DEFAULT,
  CONSTRAINT FK_Task_CalendarID FOREIGN KEY (C_CalendarID) REFERENCES T_Calendar,
  CONSTRAINT FK_Task_ParentID FOREIGN KEY (C_ParentID) REFERENCES T_Task ON DELETE SET NULL);


CREATE INDEX I_Task_Order ON T_Task (C_Order);


CREATE TABLE T_Reminder (
  C_ID INT NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_UserID INT NOT NULL,
  C_InfoTypeID INT NOT NULL,
  C_InfoID INT NOT NULL,
  C_Subject NVARCHAR(256), 
  C_Location NVARCHAR(256), 
  C_AllDayFlag BOOLEAN NOT NULL, 
  C_ScheduledTime DATETIME,
  C_DueTime DATETIME NOT NULL, 
  C_RemindTime DATETIME NOT NULL,
  C_RemindSound NVARCHAR(256),
  C_RemindActions NTEXT,
  CONSTRAINT PK_Reminder PRIMARY KEY (C_ID),
  CONSTRAINT FK_Reminder_UserID FOREIGN KEY (C_UserID) REFERENCES T_User ON DELETE CASCADE,
  CONSTRAINT FK_Reminder_InfoTypeID FOREIGN KEY (C_InfoTypeID) REFERENCES T_InfoType);


CREATE TABLE T_Diary (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL,
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_Priority SMALLINT NOT NULL,
  C_IconKind INT, 
  C_IconKind2 INT, 
  C_Date DATETIME NOT NULL, 
  C_Year SMALLINT NOT NULL, 
  C_Month SMALLINT NOT NULL, 
  C_Title NVARCHAR(256), 
  C_ContentBrief NVARCHAR(256),
  C_ContentPlainText NTEXT,
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_Content BLOB,
  C_MobileContent NTEXT,
  CONSTRAINT PK_Diary PRIMARY KEY (C_ID),
  CONSTRAINT FK_Diary_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Diary_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT);


CREATE TABLE T_StickyNote (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_Priority SMALLINT NOT NULL,
  C_ShowOnDesktop BOOLEAN NOT NULL, 
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_Content NTEXT, 
  C_Settings NTEXT,
  C_FormBounds NTEXT,
  CONSTRAINT PK_StickyNote PRIMARY KEY (C_ID),
  CONSTRAINT FK_StickyNote_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_StickyNote_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT);


CREATE TABLE T_UserStickyNote (
  C_StickyNoteID INT NOT NULL, 
  C_UserID INT NOT NULL, 
  C_ShowOnDesktop BOOLEAN NOT NULL, 
  C_Settings NTEXT,
  C_FormBounds NTEXT,
  CONSTRAINT PK_UserStickyNote PRIMARY KEY (C_StickyNoteID, C_UserID),
  CONSTRAINT FK_UserStickyNote_StickyNoteID FOREIGN KEY (C_StickyNoteID) REFERENCES T_StickyNote ON DELETE CASCADE,
  CONSTRAINT FK_UserStickyNote_UserID FOREIGN KEY (C_UserID) REFERENCES T_User ON DELETE CASCADE);


CREATE TABLE T_Note (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_Priority SMALLINT NOT NULL,
  C_Title NVARCHAR(256),
  C_ContentBrief NVARCHAR(256),
  C_ContentPlainText NTEXT,
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_Content BLOB,
  C_MobileContent NTEXT,
  CONSTRAINT PK_Note PRIMARY KEY (C_ID),
  CONSTRAINT FK_Note_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Note_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT);


CREATE TABLE T_Password (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_Priority SMALLINT NOT NULL,
  C_Title NVARCHAR(256),
  C_Location NVARCHAR(256),
  C_UserName NVARCHAR(256),
  C_AccountID NVARCHAR(256),
  C_Password NVARCHAR(256),
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_CommentPlainText NTEXT,
  C_Comment BLOB,
  C_MobileComment NTEXT,
  CONSTRAINT PK_Password PRIMARY KEY (C_ID),
  CONSTRAINT FK_Password_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_Password_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT);


CREATE TABLE T_SoftRegCode (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_Priority SMALLINT NOT NULL,
  C_SoftwareName NVARCHAR(256),
  C_SoftwareVersion NVARCHAR(256),
  C_SoftwareURL NVARCHAR(256),
  C_RegUserName NVARCHAR(256),
  C_RegCode NVARCHAR(256),
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_CommentPlainText NTEXT,
  C_Comment BLOB,
  C_MobileComment NTEXT,
  CONSTRAINT PK_SoftRegCode PRIMARY KEY (C_ID),
  CONSTRAINT FK_SoftRegCode_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_SoftRegCode_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT);


CREATE TABLE T_EmailAccount (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_Priority SMALLINT NOT NULL,
  C_EmailAddress NVARCHAR(256),
  C_WebmailURL NVARCHAR(256),
  C_IncomingHost NVARCHAR(256),
  C_IncomingPort INT, 
  C_IncomingUserName NVARCHAR(256),
  C_IncomingPassword NVARCHAR(256),
  C_IncomingUseSecurePasswordAuth BOOLEAN NOT NULL, 
  C_IncomingRequireSSL BOOLEAN NOT NULL, 
  C_OutgoingHost NVARCHAR(256),
  C_OutgoingPort INT, 
  C_OutgoingAuthenticationType SMALLINT NOT NULL, 
  C_OutgoingSameAsIncoming BOOLEAN NOT NULL, 
  C_OutgoingUserName NVARCHAR(256),
  C_OutgoingPassword NVARCHAR(256),
  C_OutgoingUseSecurePasswordAuth BOOLEAN NOT NULL, 
  C_OutgoingRequireSSL BOOLEAN NOT NULL, 
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_CommentPlainText NTEXT,
  C_Comment BLOB,
  C_MobileComment NTEXT,
  CONSTRAINT PK_EmailAccount PRIMARY KEY (C_ID),
  CONSTRAINT FK_EmailAccount_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_EmailAccount_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT);


CREATE TABLE T_FTPAccount (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_Priority SMALLINT NOT NULL,
  C_Name NVARCHAR(256),
  C_Host NVARCHAR(256),
  C_Port INT, 
  C_UserName NVARCHAR(256),
  C_Password NVARCHAR(256),
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_CommentPlainText NTEXT,
  C_Comment BLOB,
  C_MobileComment NTEXT,
  CONSTRAINT PK_FTPAccount PRIMARY KEY (C_ID),
  CONSTRAINT FK_FTPAccount_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_FTPAccount_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT);


CREATE TABLE T_FavoriteURL (
  C_ID INT NOT NULL,
  C_Creator INT DEFAULT 1 NOT NULL,
  C_Modifier INT DEFAULT 1 NOT NULL,
  C_Private BOOLEAN NOT NULL,
  C_CreationTime DATETIME NOT NULL, 
  C_LastModificationTime DATETIME NOT NULL, 
  C_GUID GUID NOT NULL, 
  C_SyncFlag SMALLINT DEFAULT 0 NOT NULL,
  C_DeletedFlag BOOLEAN NOT NULL, 
  C_AttachmentFlag BOOLEAN NOT NULL,
  C_Priority SMALLINT NOT NULL,
  C_Title NVARCHAR(256),
  C_URL NVARCHAR(256),
  C_UserName NVARCHAR(256),
  C_Password NVARCHAR(256),
  C_Folders NVARCHAR(256),
  C_Categories NVARCHAR(512),
  C_Contacts NVARCHAR(512),
  C_Attachments NTEXT,
  C_CommentPlainText NTEXT,
  C_Comment BLOB,
  C_MobileComment NTEXT,
  CONSTRAINT PK_FavoriteURL PRIMARY KEY (C_ID),
  CONSTRAINT FK_FavoriteURL_Creator FOREIGN KEY (C_Creator) REFERENCES T_User ON DELETE SET DEFAULT,
  CONSTRAINT FK_FavoriteURL_Modifier FOREIGN KEY (C_Modifier) REFERENCES T_User ON DELETE SET DEFAULT);


CREATE TRIGGER BDO_Folder BEFORE DELETE ON T_Folder
begin
  UPDATE T_Folder SET C_ParentID = NULL WHERE C_ParentID = Old.C_ID;
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(-1, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_ViewSetting WHERE C_FolderID = Old.C_ID;
  DELETE FROM T_InfoFolder WHERE C_FolderID = Old.C_ID;
end;


CREATE TRIGGER BDO_Contact BEFORE DELETE ON T_Contact
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(1, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 1 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 1 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 1 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_ContactID = Old.C_ID;
  DELETE FROM T_ContactPhoto WHERE C_ContactID = Old.C_ID;
  DELETE FROM T_ContactEvent WHERE C_ContactID = Old.C_ID;
end;


CREATE TRIGGER BUO_EventLabel BEFORE UPDATE ON T_EventLabel
when Old.C_DeletedFlag = 0 and New.C_DeletedFlag = 1
begin
  UPDATE T_Event SET C_LabelID = 1 WHERE C_LabelID = Old.C_ID;
  UPDATE T_EventItem SET C_LabelID = 1 WHERE C_LabelID = Old.C_ID;
end;


CREATE TRIGGER BDO_Event BEFORE DELETE ON T_Event
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(2, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 2 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 2 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 2 and C_InfoID = Old.C_ID;
  DELETE FROM T_ContactEvent WHERE C_EventID = Old.C_ID;
  DELETE FROM T_EventItem WHERE C_EventID = Old.C_ID;
  DELETE FROM T_Reminder WHERE C_InfoTypeID = 2 and C_InfoID = Old.C_ID;
end;


CREATE TRIGGER BUO_Event BEFORE UPDATE ON T_Event
when New.C_DeletedFlag = 1 and Old.C_DeletedFlag = 0
begin
  update T_Contact set C_RemindBirthday = 0 where C_ID = (select C_ContactID from T_ContactEvent where C_EventID = Old.C_ID) and (select C_AnniversaryType from T_ContactEvent where C_EventID = Old.C_ID) = 1;
  update T_Contact set C_RemindAnniversary = 0 where C_ID = (select C_ContactID from T_ContactEvent where C_EventID = Old.C_ID) and (select C_AnniversaryType from T_ContactEvent where C_EventID = Old.C_ID) = 2;
end;


CREATE TRIGGER ADO_EventItem AFTER DELETE ON T_EventItem
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(-2, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
end;


CREATE TRIGGER BUO_TaskLabel BEFORE UPDATE ON T_TaskLabel
when Old.C_DeletedFlag = 0 and New.C_DeletedFlag = 1
begin
  UPDATE T_Task SET C_LabelID = 1 WHERE C_LabelID = Old.C_ID;
end;


CREATE TRIGGER BDO_Task BEFORE DELETE ON T_Task
begin
  UPDATE T_Task SET C_ParentID = NULL WHERE C_ParentID = Old.C_ID;
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(3, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 3 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 3 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 3 and C_InfoID = Old.C_ID;
  DELETE FROM T_Reminder WHERE C_InfoTypeID = 3 and C_InfoID = Old.C_ID;
end;


CREATE TRIGGER ADO_Reminder AFTER DELETE ON T_Reminder
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(-3, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
end;


CREATE TRIGGER BDO_Diary BEFORE DELETE ON T_Diary
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(4, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 4 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 4 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 4 and C_InfoID = Old.C_ID;
end;


CREATE TRIGGER BDO_Note BEFORE DELETE ON T_Note
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(5, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 5 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 5 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 5 and C_InfoID = Old.C_ID;
end;


CREATE TRIGGER BDO_StickyNote BEFORE DELETE ON T_StickyNote
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(6, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 6 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 6 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 6 and C_InfoID = Old.C_ID;
  DELETE FROM T_UserStickyNote WHERE C_StickyNoteID = Old.C_ID;
end;


CREATE TRIGGER BDO_Password BEFORE DELETE ON T_Password
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(7, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 7 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 7 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 7 and C_InfoID = Old.C_ID;
end;


CREATE TRIGGER BDO_SoftRegCode BEFORE DELETE ON T_SoftRegCode
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(8, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 8 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 8 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 8 and C_InfoID = Old.C_ID;
end;


CREATE TRIGGER BDO_EmailAccount BEFORE DELETE ON T_EmailAccount
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(9, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 9 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 9 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 9 and C_InfoID = Old.C_ID;
end;


CREATE TRIGGER BDO_FTPAccount BEFORE DELETE ON T_FTPAccount
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(10, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 10 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 10 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 10 and C_InfoID = Old.C_ID;
end;


CREATE TRIGGER BDO_FavoriteURL BEFORE DELETE ON T_FavoriteURL
begin
  INSERT INTO T_DeleteLog (C_InfoTypeID,C_InfoGUID,C_DeletedTime) VALUES(11, Old.C_GUID, cast(strftime('%s','now','localtime') as integer));
  DELETE FROM T_InfoFolder WHERE C_InfoTypeID = 11 and C_InfoID = Old.C_ID;
  DELETE FROM T_Attachment WHERE C_InfoTypeID = 11 and C_InfoID = Old.C_ID;
  DELETE FROM T_InfoContact WHERE C_InfoTypeID = 11 and C_InfoID = Old.C_ID;
end;
